 
/*
 * FILE:
 * Sheet.c
 *
 * FUNCTION:
 * create reports 
 *
 * DESIGN:
 * Much work needs to be done.  
 *  o First, we need a chart of accounts, so that we
 *    can have a rational sorting order.  
 *  o We need a way of stripping out the html from the code,
 *    and having some sort of template instead.
 *  o We need some way of defining which routine gets invoked 
 *    depending on which link gets pressed.
 *
 * HISTORY:
 * Created by Linas Vepstas October 1998
 */

#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "Account.h"
#include "Group.h"
#include "util.h"

extern AccountGroup *topgroup;

/* ======================================================= */

static FILE *
StartUp (void) 
{
  int fd;
  char tmpfilename[200];
  FILE *fh;

  
  /* create a tmp file with a bit of privacy */
  strcpy (tmpfilename, "/tmp/gncXXXXXX");
  mktemp (tmpfilename);
  strcat (tmpfilename, ".html");
  fd = open (tmpfilename, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
  if (0 > fd) {
    ERROR ();
    fprintf (stderr, "Error: can't open file %s\n", tmpfilename);
    return NULL;
  }

  fh = fdopen (fd, "w+");
  if (!fh) {
    ERROR ();
    fprintf (stderr, "Error: can't open file %s\n", tmpfilename);
    return NULL;
  }
  unlink (tmpfilename);  /* make sure file is deleted even if we crash */

  return fh;
}
  
/* ======================================================= */

static char * 
FinishUp (FILE *fh) 
{
  int fd, rc;
  char * text = NULL;
  size_t size;

  /* make sure that file buffers are fluished ... */
  fflush (fh);

  /* Find size: */
  fd = fileno (fh);
  size = lseek( fd, 0, SEEK_END );
  lseek( fd, 0, SEEK_SET );

  text = (char *) malloc ((size+1)*sizeof (char));

  /* read in file */
  rc = read (fd,text,size);
  if (-1 == rc) {
    free (text);
    text = NULL;
  }
  text[size] = 0x0;
  fclose (fh);
  return text;
}

/* ======================================================= */

#define OUT fprintf (fh,

/* ======================================================= */

char * 
ReportBalance (void) 
{
  char * text;
  FILE *fh;
  int i;
  Account * acc;

  fh = StartUp ();
  if (!fh) return NULL;
  
  /* print report header */
  OUT "<html><body>\n"
      "This is a Test, This is Only a Test<br>"
      "If this were for real, you wouldn't be reading this.<p>"
      "<table>\n"
      "<caption>Balance Sheet</caption>"
      "<tr><th>Account Name<th>Type<th>Balance"
      );

  i=0;
  acc = xaccGroupGetAccount (topgroup, i);
  while (acc) {
    int type;
    double baln;

    type = xaccAccountGetType(acc);

    baln = xaccAccountGetBalance(acc);
    baln += xaccGroupGetBalance (xaccAccountGetChildren(acc));

    /* reverse the sign on income and expense accounts */
    if ((EXPENSE == type) || (INCOME  == type )) {
      baln = -baln;
    }

    OUT "<tr>\n"
        "<td>%s<td>%s<td>%11.2f\n", 
            xaccAccountGetName (acc), 
            xaccAccountGetTypeStr (type),
            baln);
    i++; acc = xaccGroupGetAccount (topgroup, i);
  }
  OUT "</table>\n"
      "</body></html>\n");

  text = FinishUp (fh);
  return text;
}

